//----------------------------------------------------------------------------
//
// Vision Advance Technology - Software Development Kit
// Copyright (c) 2014-2024, Vision Advance Technology Inc.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.
//

#ifndef _HALREG_DEFINE_
#define _HALREG_DEFINE_

#define HALREG_ERR_SUCCESS              0x00000000
#define HALREG_EMPTY_HAL				0x00000000

#define HALREG_ERR_TAG                  0x80000000
#define HALREG_ERR_MODULATOR			0x80100000
#define HALREG_ERR_BROADCAST			0x80200000
#define HALREG_ERR_ENCODER				0x80400000
#define HALREG_ERR_STREAM				0x80800000
#define HALREG_ERR_OUTPUT				0x81000000
#define HALREG_ERR_MUX					0x82000000
#define HALREG_ERR_R2					0x84000000
#define HALREG_ERR_TRANSFORM			0x88000000

#define MODPARAM_ERR_BW					(HALREG_ERR_MODULATOR | 0x00)
#define MODPARAM_ERR_CONSTELLATION		(HALREG_ERR_MODULATOR | 0x01)
#define MODPARAM_ERR_NOTSUPPORT			(HALREG_ERR_MODULATOR | 0x02)
#define MODPARAM_ERR_SYNCFRAME_1		(HALREG_ERR_MODULATOR | 0x03)
#define MODPARAM_ERR_SYNCFRAME_2		(HALREG_ERR_MODULATOR | 0x04)
#define MODPARAM_ERR_T2EXTCARRIER		(HALREG_ERR_MODULATOR | 0x05)
#define MODPARAM_ERR_T2FFT32K			(HALREG_ERR_MODULATOR | 0x06)
#define MODPARAM_ERR_T216KGI			(HALREG_ERR_MODULATOR | 0x07)
#define MODPARAM_ERR_T2FECCR			(HALREG_ERR_MODULATOR | 0x08)
#define MODPARAM_ERR_T2SYMBOLNUM		(HALREG_ERR_MODULATOR | 0x09)
#define MODPARAM_ERR_T2FECBLOCKNUM		(HALREG_ERR_MODULATOR | 0x0A)
#define MODPARAM_ERR_HWNOTSUPPORT		(HALREG_ERR_MODULATOR | 0x0B)
#define MODPARAM_ERR_SUCCESS			HALREG_ERR_SUCCESS

#define ENCPARAM_ERR_CODECSUPPORT		(HALREG_ERR_ENCODER | 0x00)
#define ENCPARAM_ERR_VIDEO				(HALREG_ERR_ENCODER | 0x01)
#define ENCPARAM_ERR_AUDIO				(HALREG_ERR_ENCODER | 0x02)
#define ENCPARAM_ERR_FRAMERATE			(HALREG_ERR_ENCODER | 0x03)
#define ENCPARAM_ERR_QUANT				(HALREG_ERR_ENCODER | 0x04)
#define ENCPARAM_ERR_GOPLEN				(HALREG_ERR_ENCODER | 0x05) 
#define ENCPARAM_ERR_PSIMODE			(HALREG_ERR_ENCODER | 0x06) 

#define BROADCAST_ERR_BADCMD			(HALREG_ERR_BROADCAST | 0x00)
#define BROADCAST_ERR_HWFAIL			(HALREG_ERR_BROADCAST | 0x01)
#define BROADCAST_ERR_HWSUPPORT			(HALREG_ERR_BROADCAST | 0x02)
#define BROADCAST_ERR_BADSTATUS			(HALREG_ERR_BROADCAST | 0x03)
#define BROADCAST_ERR_RFMIXER			(HALREG_ERR_BROADCAST | 0x04)
#define BROADCAST_ERR_BADPARAM			(HALREG_ERR_BROADCAST | 0x05)
#define BROADCAST_ERR_UNKNOWN			(HALREG_ERR_BROADCAST | 0x80)

#define MUX_ERR_NOLICENSED				(HALREG_ERR_MUX | 0x00)
#define MUX_ERR_LOADRULE				(HALREG_ERR_MUX | 0x01)

#define STREAM_ERR_SOURCE				(HALREG_ERR_STREAM | 0x00)	
#define STREAM_ERR_MODE					(HALREG_ERR_STREAM | 0x01)

#define OUTPUT_ERR_MODE					(HALREG_ERR_OUTPUT | 0x00)

#define R2_ERR_FREQ						(HALREG_ERR_R2 | 0x00)
#define R2_ERR_TUNEHEADER				(HALREG_ERR_R2 | 0x01)
#define R2_ERR_RULEPARAM				(HALREG_ERR_R2 | 0x02)

#define TRANSFORM_ERR_TIMEOUT			(HALREG_ERR_TRANSFORM | 0x01)

#define HALREG_RANGE_BASE_START			0x0000
#define HALREG_RANGE_BASE_END			0x0100

#define HALREG_RANGE_APP_0_START		0x600
#define HALREG_RANGE_APP_0_END			0x800
#define HALREG_RANGE_APP_1_START		0x800
#define HALREG_RANGE_APP_1_END			0x4000

#endif
